#!/usr/bin/env node
/**
 * Copyright (c) 2015-present, Facebook, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

'use strict';

const fs = require('fs');
const path = require('path');
const spawn = require('react-dev-utils/crossSpawn');
const paths = require('../config/paths');
const args = process.argv.slice(2);

const scriptIndex = args.findIndex(
  x => x === 'build' || x === 'start' || x === 'test'
);
const script = scriptIndex === -1 ? args[0] : args[scriptIndex];
const nodeArgs = scriptIndex > 0 ? args.slice(0, scriptIndex) : [];
const useTS = fs.existsSync(path.join(paths.appPath, 'tsconfig.json'));

switch (script) {
  case 'build':
  case 'test': {
    const result = spawn.sync(
      'node',
      nodeArgs
        .concat(require.resolve('../scripts/' + script))
        .concat(args.slice(scriptIndex + 1)),
      { stdio: 'inherit' }
    );
    if (result.signal) {
      if (result.signal === 'SIGKILL') {
        console.log(
          'The build failed because the process exited too early. ' +
          'This probably means the system ran out of memory or someone called ' +
          '`kill -9` on the process.'
        );
      } else if (result.signal === 'SIGTERM') {
        console.log(
          'The build failed because the process exited too early. ' +
          'Someone might have called `kill` or `killall`, or the system could ' +
          'be shutting down.'
        );
      }
      process.exit(1);
    }
    process.exit(result.status);
    break;
  }
  case 'start': {
    process.env.BABEL_ENV = 'development';
    process.env.NODE_ENV = 'development';

    const host = process.env.HOST || '0.0.0.0';
    const port = process.env.PORT || '3000';
    const config = path.resolve(__dirname, '..', 'config/webpack.config.js');
    const devServerArgs = args.slice(1).concat([
      '--debug',
      '--output-pathinfo',
      '--history-api-fallback',
      '--hot',
      '--inline',
      '--progress',
      '--colors',
      '--host', host,
      '--port', port,
      '--config', config,
    ]);
    const result = spawn.sync(
      'webpack-dev-server',
      devServerArgs,
      { stdio: 'inherit' }
    );
    if (result.error) {
      throw result.error;
    }
    process.exit(result.status);
    break;
  }
  case 'lint': {
    const eslintConfig = path.resolve(__dirname, '..', 'config/.eslintrc');
    const commands = [
      `eslint src/**/*.{js,jsx} -c ${eslintConfig}`,
      `eslint e2e/**/*.{js,jsx} -c ${eslintConfig}`,
      `stylelint 'src/**/*.less' 'styles/**/*.less' --color --syntax=less --formatter=string`,
    ];
    if (useTS) {
      commands.push(
        `tslint --project ./tsconfig.json`,
        `tslint --project ./e2e/tsconfig.json`,
      )
    }

    const result = spawn.sync(
      commands.join(' && '),
      [],
      { stdio: 'inherit', shell: true }
    );
    if (result.error) {
      throw result.error;
    }
    process.exit(result.status);
    break;
  }
  case 'check': {
    if (!useTS) {
      process.exit(0);
      break;
    }
    const result = spawn.sync(
      [
        `tsc --project ./tsconfig.json --noEmit --pretty`,
        `tsc --project ./e2e/tsconfig.json --noEmit --pretty`,
      ].join(' && '),
      [],
      { stdio: 'inherit', shell: true }
    );
    if (result.error) {
      throw result.error;
    }
    process.exit(result.status);
    break;
  }
  case 'e2e': {
    const config = path.resolve(__dirname, '..', 'config/protractor.conf.js');
    const isDebug = args.includes('--debug');
    const isDocker = args.includes('--docker');
    const e2eEnv = args.filter(key => /^E2E_/.test(key));
    const e2eArgs = isDocker
      ? e2eEnv.concat(['protractor', config]).filter(Boolean)
      : [
        'run-p', '--race',
        'chromedriver --port=9511 --silent',
        isDebug ? '' : 'serve -p 3000',
        `protractor ${config}`
      ].filter(Boolean);

    const result = spawn.sync('cross-env', e2eArgs, { stdio: 'inherit' });
    if (result.error) {
      throw result.error;
    }
    process.exit(result.status);
    break;
  }
  default:
    console.log('Unknown script "' + script + '".');
    console.log('Perhaps you need to update react-scripts-ts?');
    break;
}
