const fs = require('fs');
const path = require('path');
const webpack = require('webpack');

const HtmlWebpackPlugin = require('html-webpack-plugin');
const ExtractTextPlugin = require('extract-text-webpack-plugin');
const CopyWebpackPlugin = require('copy-webpack-plugin');
const BundleAnalyzerPlugin = require('webpack-bundle-analyzer').BundleAnalyzerPlugin;

const getClientEnvironment = require('./env');
const paths = require('./paths');

const publicPath = paths.servedPath;
const publicUrl = publicPath.slice(0, -1);
const env = getClientEnvironment(publicUrl);

const isJsonOut = process.argv.indexOf('--json') >= 0;
const isProdMode = env.raw.NODE_ENV === "production";
const isVendorModule = ({resource}) => resource ? resource.includes('/node_modules/') : false;

const outPath = paths.appBuild;
const sourcePath = paths.appSrc;
const staticPath = paths.appPublic;

let aliases = {};
if (env.raw.REACT_APP_ALIASES) {
  try {
    const parsedAliases = JSON.parse(env.raw.REACT_APP_ALIASES);
    aliases = Object
      .keys(parsedAliases)
      .reduce((result, key) => ({
        ...result,
        [key]: path.resolve(paths.appPath, parsedAliases[key])
      }), {});
  } catch (e) {}
}

const indexEntries = ['./index.tsx', './index.jsx', './index.js']
  .filter((it) => fs.existsSync(path.join(paths.appSrc, it)));
const indexMocksEntries = ['./index-mocks.tsx', './index-mocks.jsx', './index-mocks.js']
  .filter((it) => fs.existsSync(path.join(paths.appSrc, it)));
const babelConfig = getBabelConfig();

module.exports = {
  context: sourcePath,
  entry: {
    index: [
      '@babel/polyfill',
      'react-hot-loader/patch',
    ].concat(indexEntries),
    mocks: [
      '@babel/polyfill',
      'react-hot-loader/patch',
    ].concat(indexMocksEntries),
  },
  output: {
    path: outPath,
    publicPath: '',
    filename: isProdMode ? '[name].[chunkhash:8].js' : '[name].[hash:8].js',
  },
  target: 'web',
  resolve: {
    extensions: ['.js', '.jsx', '.ts', '.tsx'],
    mainFields: ['browser', 'main'],
    alias: {
      src: sourcePath,
      static: staticPath,
      ...aliases,
    },
  },
  module: {
    loaders: [
      {
        test: /\.jsx?$/,
        use: [
          {
            loader: 'babel-loader',
            options: {
              ...babelConfig,
              plugins: [
                ...babelConfig.plugins,
                ...isProdMode ? [] : ['react-hot-loader/babel'],
              ],
            },
          },
        ],
      },
      {
        test: /\.tsx?$/,
        use: [
          {
            loader: 'awesome-typescript-loader',
            options: {
              silent: isJsonOut,
              useBabel: true,
              babelOptions: {
                plugins: [
                  'react-hot-loader/babel',
                ],
              },
            },
          },
        ],
      },
      {
        test: /\.tsx?$/,
        enforce: 'post',
        use: env.raw.REACT_APP_COVERAGE
          ? [{loader: 'istanbul-instrumenter-loader', options: {esModules: true}}]
          : [],
      },
      {
        test: /\/static\/translations\/.+\.po$/,
        use: [
          {
            loader: 'json-loader',
          },
          {
            loader: 'po-loader',
          },
        ],
      },
      {
        test: /\/static\/mockfiles\/.+$/,
        use: 'base64-loader',
      },
      {
        test: /\.(le|c)ss$/,
        use: ExtractTextPlugin.extract({
          fallback: 'style-loader',
          use: [
            {
              loader: require.resolve('./webpack.css-sm-fix.js'),
              options: {
                enabled: !isProdMode,
                sourceRoot: paths.appPath,
                extensions: ['.css', '.less'],
              },
            },
            {
              loader: 'css-loader',
              options: {
                modules: false,
                sourceMap: true,
              },
            },
            {
              loader: 'less-loader',
              options: {
                paths: [paths.appPath],
                sourceMap: true,
              },
            },
          ],
        }),
      },
      {
        test: /\.html$/,
        use: 'html-loader',
        exclude: staticPath,
      },
      {
        test: /\.(png|gif)$/,
        loader: 'url-loader',
        options: {name: 'images/[name].[ext]', limit: 10000},
        exclude: staticPath,
      },
      {
        test: /\.jpg$/,
        loader: 'file-loader',
        options: {name: 'images/[name].[ext]'},
        exclude: staticPath,
      },
      {
        test: /\.(eot|svg|ttf|woff|woff2)$/,
        loader: 'file-loader',
        options: {name: 'fonts/[name].[ext]'},
        exclude: staticPath,
      },
    ],
  },
  plugins: getWebpackPlugins(),
  devServer: {
    contentBase: sourcePath,
    hot: true,
    stats: {
      warnings: false,
    },
  },
  node: {
    // workaround for webpack-dev-server issue
    // https://github.com/webpack/webpack-dev-server/issues/60#issuecomment-103411179
    fs: 'empty',
    net: 'empty',
  },
  devtool: isProdMode ? "source-map" : "cheap-module-source-map",
};

function getWebpackPlugins() {
  let libraries = {};
  if (env.raw.REACT_APP_LIBRARIES) {
    try {
      libraries = JSON.parse(env.raw.REACT_APP_LIBRARIES);
      if (!libraries) {
        libraries = {};
      }
    } catch (e) {
    }
  }

  const plugins = [
    new webpack.NamedModulesPlugin(),
    new webpack.ProvidePlugin(libraries),
    new webpack.DefinePlugin(env.stringified),
    new webpack.optimize.CommonsChunkPlugin({
      name: 'vendors',
      chunks: ['index'],
      minChunks: isVendorModule,
    }),
    new webpack.optimize.CommonsChunkPlugin({
      name: 'vendors-mocks',
      chunks: ['mocks'],
      minChunks: isVendorModule,
    }),

    new webpack.optimize.CommonsChunkPlugin({
      name: 'common',
      chunks: ['index', 'mocks'],
    }),
    new webpack.optimize.CommonsChunkPlugin({
      name: 'vendors',
      chunks: ['vendors', 'vendors-mocks'],
    }),

    new webpack.optimize.AggressiveMergingPlugin(),
    new ExtractTextPlugin({
      filename: '[name].[chunkhash:8].css',
      disable: !isProdMode,
    }),
    new HtmlWebpackPlugin({
      template: 'index.html',
      filename: 'index.html',
      inject: true,
      minify: false,
      chunks: ['vendors', 'common', 'index'],
      chunksSortMode: 'manual',
    }),
    new HtmlWebpackPlugin({
      template: 'index.html',
      filename: 'mocks.html',
      inject: true,
      minify: false,
      chunks: ['vendors', 'vendors-mocks', 'common', 'mocks', 'index'],
      chunksSortMode: 'manual',
    }),
    new CopyWebpackPlugin([
      {
        from: path.join(staticPath, 'favicon.ico'),
        to: 'favicon.ico',
      },
      {
        from: path.join(staticPath, 'images'),
        to: 'images',
      },
    ]),
  ];
  if (isProdMode) {
    plugins.push(new webpack.optimize.UglifyJsPlugin({sourceMap: true}));
  }
  if (process.env.REACT_APP_ANALYZE) {
    plugins.push(new BundleAnalyzerPlugin());
  }
  return plugins;
}

function getBabelConfig() {
  const defaultBabelConfig = JSON.parse(
    fs.readFileSync(path.resolve(__dirname, '.babelrc'), 'utf-8').toString()
  );
  let customerBabelConfig = {
    plugins: [],
  };
  try {
    customerBabelConfig = JSON.parse(
      fs.readFileSync(path.join(paths.appPath, '.babelrc'), 'utf-8').toString()
    );
  } catch (e) {
  }
  return {
    ...defaultBabelConfig,
    ...customerBabelConfig,
    plugins: [
      ...defaultBabelConfig.plugins,
      ...(customerBabelConfig.plugins || []),
    ],
    presets: [
      ...defaultBabelConfig.presets,
      ...(customerBabelConfig.presets || []),
    ],
  };
}
